@@ -17,5 +17,5 @@ func TestNetworkPolicyRender(t *testing.T) {
 
 	assert.NoError(t, c.Render(load(t, "np"), "", &r))
 	assert.Equal(t, "default/fred", r.ID)
-	assert.Equal(t, model1.Fields{"default", "fred", "ns:app=blee,po:app=fred", "TCP:6379", "172.17.0.0/16[172.17.1.0/24,172.17.3.0/24...]", "", "TCP:5978", "10.0.0.0/24"}, r.Fields[:8])
+	assert.Equal(t, model1.Fields{"default", "fred", "app=nginx", "ns:app=blee,po:app=fred", "TCP:6379", "172.17.0.0/16[172.17.1.0/24,172.17.3.0/24...]", "", "TCP:5978", "10.0.0.0/24"}, r.Fields[:9])
 }
